/*LH created file to merge both Send Email and Get Time
 * Identical file for F5529, FR5969, and FR6989
 * Major changes to make it work with function calls from main
 * Commented out almost all Loop Forever which just hangs my whole application
 *
 * email.c - function implementation to send email
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 * All rights reserved. Property of Texas Instruments Incorporated.
 * Restricted rights to use, duplicate or disclose this code are
 * granted through contract.
 *
 * The program may not be used without the written permission of
 * Texas Instruments Incorporated or against the terms and conditions
 * stipulated in the agreement under which this program has been supplied,
 * and under no circumstances can it be used with non-TI connectivity device.
 *
 */

/*
 * Protocol Name     -  Simple Mail Transfer Protocol
 * Protocol Overview -  The objective of the Simple Mail Transfer Protocol (SMTP) is to
 *                      transfer mail reliably and efficiently.SMTP is independent of the
 *                      particular transmission subsystem and requires only a reliable
 *                      ordered data stream channel.
 *                      Refer: https://www.ietf.org/rfc/rfc2821.txt
 */
volatile unsigned short usiEmail1OrTime2 = 0; //In the function call set to 1 for email, set to 2 for get-time
//Same function call can be used but inside the function the specific statements are for email or get time
//I do not want to create a similar function with a different name as I do not know where else in simplelink it may be calling these functions

#include <emailAndGetTime.h>
#include "base64.h"
#include "config.h"
#include "sl_common.h"

#include "myGPIO.h" //8-23-2020 LH added

/*NetApp Email protocol types */
#define SL_NET_APP_SMTP_PROTOCOL        (1)
#define INVALID_SOCKET_DESC             0xFFFFFFFF

/* Strings used to configure the Email msg in proper format */
const _u8 smtp_helo[] = {'H','E','L','O','\0'};
const _u8 smtp_mail_from[]={'M','A','I','L',' ','F','R','O','M',':',' ','\0'};
const _u8 smtp_rcpt[] = {'R','C','P','T',' ','T','O',':',' ','\0'};
const _u8 smtp_data[] = "DATA";
const _u8 smtp_crlf[] = "\r\n";
const _u8 smtp_dcrlf[] = "\r\n\r\n";
const _u8 smtp_subject[] = "Subject: ";
const _u8 smtp_to[] = "To: ";
const _u8 smtp_from[] = "From: ";
/* <CRLF>.<CRLF> Terminates the data portion */
const _u8 smtp_data_end[] = {'\r','\n','.','\r','\n','\0'};
const _u8 smtp_quit[] = {'Q','U','I','T','\r','\n','\0'};

/* Return Codes */
const _u8 smtp_code_ready[] = {'2','2','0','\0'};
const _u8 smtp_ok_reply[] = {'2','5','0','\0'};
const _u8 smtp_intermed_reply[] = {'3','5','4','\0'};
const _u8 smtp_auth_reply[] = {'3','3','4','\0'};
const _u8 smtp_auth_success[] = {'2','3','5','\0'};

/* states for smtp state machine */
typedef enum
{
    smtpINACTIVE = 0,
    smtpINIT,
    smtpHELO,
    smtpAUTH,
    smtpFROM,
    smtpRCPT,
    smtpDATA,
    smtpMESSAGE,
    smtpQUIT,
    smtpERROR
}_SlsmtpStatus_e;

/* Initialize SMTP State Machine */
_u16 g_smtpStatus = smtpINIT;

_u16 g_EmailSetStatus = 0;

/* error handling flags for smtp state machine */
typedef enum
{
    smtpNOERROR = 0,
    atINIT,
    atHELO,
    atAUTH,
    atFROM,
    atRCPT,
    atDATA,
    atMESSAGE,
    atQUIT
}_SlsmtpERROR_e;

/* Initialize Error handling flag */
_u16 g_smtpErrorInfo = smtpNOERROR;
_i32 smtpSocket;
_u8 g_cmdBuf[SMTP_BUF_LEN];
_u8 buf[SMTP_BUF_LEN];
_u8 basekey1[BASEKEY_LEN];
_u8 basekey2[BASEKEY_LEN];
_u8 message[MAX_MESSAGE_LEN];

SlNetAppEmailOpt_t g_EmailOpt;
SlNetAppSourceEmail_t g_Email;
SlNetAppSourcePassword_t g_SourcePass;
SlNetAppDestination_t g_Destination;
SlNetAppEmailSubject_t g_Subject;

_u8 email_rfc[MAX_EMAIL_RCF_LEN];

static _i32 _smtpConnect(void);
static _i32 _smtpSend(void);
static void _smtpHandleERROR(_u8 * servermessage);
static _i32 _sendSMTPCommand(_i32 socket, _u8 * cmd, _u8 * cmdparam, _u8 * respBuf);
static void _generateBase64Key(_u8 * basekey1, _u8 * basekey2);


/*!
    \brief This function sets the email parameters

    \param[in]      command -   Command send for processing
    \param[in]      pValueLen - Length of data to be processed
    \param[in]      pValue -     Data to be processed

    \return         0 for success, -1 otherwise

    \note

    \warning
*/
_i32 sl_NetAppEmailSet(_u8 command ,_u8 pValueLen,
                      _u8 *pValue)
{

    SlNetAppEmailOpt_t* pEmailOpt = 0;
    SlNetAppSourceEmail_t* pSourceEmail = NULL;
    SlNetAppSourcePassword_t* pSourcePassword = NULL;
    SlNetAppDestination_t* pDestinationEmail = NULL;
    SlNetAppEmailSubject_t* pSubject = NULL;

    switch (command)
    {
      case NETAPP_ADVANCED_OPT:
        pEmailOpt = (SlNetAppEmailOpt_t*)pValue;

        g_EmailOpt.Port = pEmailOpt->Port;
        g_EmailOpt.Family = pEmailOpt->Family;
        g_EmailOpt.SecurityMethod = pEmailOpt->SecurityMethod;
        g_EmailOpt.SecurityCypher = pEmailOpt->SecurityCypher;
        g_EmailOpt.Ip = pEmailOpt->Ip;

        g_EmailSetStatus+=1;
        CLI_Write(" Line 159 email port, security \n\r"); //2-16-2023 added
//        CLI_Write(" \n Line 159 email port: %d\n\", Port); //Gives errors
        break;
// See Thermostat Controller code in mytimers.c; works there but not here
//    printf("\nMessage Body is: %d\n", usiGMTOffsetHrSet); //%d is an integer
//    CLI_Write("\nMessage Body is: %s\n", LHemailBody);
      case NETAPP_SOURCE_EMAIL:
        pSourceEmail = (SlNetAppSourceEmail_t*)pValue;
        pal_Memset(g_Email.Username, '\0', MAX_USERNAME_LEN);
        pal_Memcpy(g_Email.Username, pSourceEmail->Username, pValueLen);

        g_EmailSetStatus+=2;
        CLI_Write(" Line 171 email Username \n\r"); //2-16-2023 added
        break;

      case NETAPP_PASSWORD:
        pSourcePassword = (SlNetAppSourcePassword_t*)pValue;
        pal_Memset(g_SourcePass.Password, '\0', MAX_PASSWORD_LEN);
        pal_Memcpy(g_SourcePass.Password, pSourcePassword->Password, pValueLen);

        g_EmailSetStatus+=4;
        CLI_Write(" Line 180 email Password \n\r"); //2-16-2023 added
        break;

      case NETAPP_DEST_EMAIL:
        pDestinationEmail=(SlNetAppDestination_t*)pValue;
        pal_Memset(g_Destination.Email, '\0', MAX_DEST_EMAIL_LEN);
        pal_Memcpy(g_Destination.Email, pDestinationEmail->Email, pValueLen);

        g_EmailSetStatus+=8;
        CLI_Write(" Line 189 email Destination \n\r"); //2-16-2023 added
        break;

      case NETAPP_SUBJECT:
        pSubject=(SlNetAppEmailSubject_t*)pValue;
        pal_Memset(g_Subject.Value, '\0', MAX_SUBJECT_LEN);
        pal_Memcpy(g_Subject.Value, pSubject->Value, pValueLen);

        g_EmailSetStatus+=16;
        CLI_Write(" Line 198 email Subject \n\r"); //2-16-2023 added
        break;

      case NETAPP_MESSAGE:
        if(pValueLen > (MAX_MESSAGE_LEN - 1))
        {
            ASSERT_ON_ERROR(EMAIL_SET_INVALID_MESSAGE);
        }
        pal_Memset(message, '\0', MAX_MESSAGE_LEN);
        pal_Memcpy(message ,pValue, pValueLen);
        CLI_Write(" Line 208 email Message Body \n\r"); //2-16-2023 added
        break;

      default:
        CLI_Write((_u8*)"\n\rLine209 Error:Default case\n\r");
        ASSERT_ON_ERROR(EMAIL_SET_INVALID_CASE);
    }

    return SUCCESS;
}

/*!
    \brief Create a secure socket and connects to SMTP server

    \param[in]      none
    \return         0 if success and negative in case of error
    \note
    \warning
*/
_i32 sl_NetAppEmailConnect()
{
    if (!(g_EmailSetStatus >= 0x07))
    {
        CLI_Write((_u8*)"\n\rLine231 Error:Email and Subject is not configured\n\r");
        ASSERT_ON_ERROR(EMAIL_CONNECT_INVALID_CONFIURATION);
    }

    return _smtpConnect();
}

/*!
    \brief Check the connection status and sends the Email

    \param[in]      none

    \return         0 if success otherwise -1

    \note

    \warning
*/
_i32 sl_NetAppEmailSend()
{
    _i32 retVal = -1;
    retVal = _smtpSend();
    if(retVal < 0)
        sl_Close(smtpSocket);

    return retVal;
}


/*!
    \brief Creates a secure socket and connects to SMTP server

    \param[in]      none

    \return         0 if success and negative in case of error

    \note

    \warning
*/
static _i32 _smtpConnect(void)
{
    SlSockAddrIn_t  LocalAddr;
    SlTimeval_t     tTimeout;
    _i32           cipher = 0;
    _i32           LocalAddrSize = 0;
    _i8            method = 0;
    _i32            Status = 0;

    LocalAddr.sin_family = g_EmailOpt.Family;
    LocalAddr.sin_port = sl_Htons(g_EmailOpt.Port);
    LocalAddr.sin_addr.s_addr = sl_Htonl(g_EmailOpt.Ip);
    LocalAddrSize = sizeof(SlSockAddrIn_t);

    /* If TLS is required */
    if(g_EmailOpt.SecurityMethod <= 5)
    {
        /* Create secure socket */
        smtpSocket = sl_Socket(SL_AF_INET, SL_SOCK_STREAM, SL_SEC_SOCKET);
        ASSERT_ON_ERROR(smtpSocket);

        tTimeout.tv_sec = 10;
        tTimeout.tv_usec = 90000;
        Status = sl_SetSockOpt(smtpSocket, SOL_SOCKET, SL_SO_RCVTIMEO,
                               &tTimeout, sizeof(SlTimeval_t));
        ASSERT_ON_ERROR(Status);

        method = g_EmailOpt.SecurityMethod;
        cipher = g_EmailOpt.SecurityCypher;

        CLI_Write(" Line 301 In _smtpConnect after if TLS create secure socket \n\r");

        /* Set Socket Options that were just defined */
        Status = sl_SetSockOpt(smtpSocket, SL_SOL_SOCKET, SL_SO_SECMETHOD,
                               &method, sizeof(method));
        CLI_Write(" Line 306 In _smtpConnect Set Socket \n\r");
        if( Status < 0 )
        {
            sl_Close(smtpSocket);
            ASSERT_ON_ERROR(Status);
            CLI_Write(" Line 311 In _smtpConnect sl_Close smtpSocket step 1 \n\r");
        }
        Status = sl_SetSockOpt(smtpSocket, SL_SOL_SOCKET, SL_SO_SECURE_MASK,
                               &cipher, sizeof(cipher));
        if( Status < 0 )
        {
            sl_Close(smtpSocket);
            ASSERT_ON_ERROR(Status);
            CLI_Write(" Line 319 In _smtpConnect sl_Close smtpSocket step 2 \n\r");
        }
    }
    /* If no TLS required */
    else
    {
        /* Create socket */
        smtpSocket = sl_Socket(SL_AF_INET, SL_SOCK_STREAM, SL_IPPROTO_TCP);
        ASSERT_ON_ERROR(smtpSocket);

        CLI_Write(" Line 329 In _smtpConnect after if NO TLS required create socket \n\r");
    }

    /* connect to socket */
    CLI_Write(" Line 333 In _smtpConnect before connect to Socket \n\r");
    Status = sl_Connect(smtpSocket, (SlSockAddr_t *)&LocalAddr, LocalAddrSize);
    CLI_Write(" Line 335 In _smtpConnect after connect to Socket \n\r"); //2-16-2023 this prints

    if((Status < 0) && (SL_ESECSNOVERIFY != Status))
    {
        CLI_Write(" Line 339 In _smtpConnect if Status < 0 && SL_ESECSNOVERIFY != Status \n\r");  //2-16-2023 this prints
// **************************************************************
// Jan 30, 2023 I do get the above CLI write, but then no more
// Appears that Connect to Socket is NOT successful
// Next write I get is at line 914
// **************************************************

        ASSERT_ON_ERROR(Status);
        CLI_Write(" Line 347 In _smtpConnect Status after connect to socket \n\r"); //2-16-2023 do not get this
    }
    CLI_Write(" Line 349 In _smtpConnect right before return SUCCESS \n\r"); //2-16-2023 do not get this
    return SUCCESS;
}

/*!
    \brief This function will send the email

    \param[in]      none

    \return         0 if success otherwise -1

    \note

    \warning
*/
static _i32 _smtpSend(void)
{
    _i16 exit = 0;
    _i32 retVal = -1;

    /* If socket has been opened, check for acknowledge from SMTP server */
    while(exit == 0)
    {
        switch(g_smtpStatus)
        {
            /*
             * An SMTP session is initiated when a client opens a connection to
             * a server and the server responds with an opening message.
             */
            case smtpINIT:
                /* Create buffer, Read so we can check for 220 'OK' from server */
                pal_Memset(buf, 0, SMTP_BUF_LEN);

                retVal = sl_Recv(smtpSocket, buf, sizeof(buf), 0);
                if(retVal <= 0)
                {
                    ASSERT_ON_ERROR(TCP_RECV_ERROR);
                }

                /* If buffer has 220, set state to HELO */
                if(buf[0] == smtp_code_ready[0] &&
                   buf[1] == smtp_code_ready[1] &&
                   buf[2] == smtp_code_ready[2])
                {
                    g_smtpStatus = smtpHELO;
                }
                /* Else error, set state to ERROR */
                else
                {
                    g_smtpStatus = smtpERROR;
                    g_smtpErrorInfo = atINIT;
                }
            break;

            /*
             * The client normally sends the EHLO command to the
             * server, indicating the client's identity.  In addition to opening the
             * session, use of EHLO indicates that the client is able to process
             * service extensions and requests that the server provide a list of the
             * extensions it supports.
             */
            case smtpHELO:
                retVal = _sendSMTPCommand(smtpSocket, "HELO localhost", NULL, buf);
                ASSERT_ON_ERROR(retVal);

                /* If response has 250, set state to AUTH */
                if(buf[0] == smtp_ok_reply[0] &&
                   buf[1] == smtp_ok_reply[1] &&
                   buf[2] == smtp_ok_reply[2])
                {
                    g_smtpStatus = smtpAUTH;
                }
                /* Else error, set state to ERROR */
                else
                {
                    g_smtpStatus = smtpERROR;
                    g_smtpErrorInfo = atHELO;
                }
            break;

            /* The client sends the AUTH command with SASL mechanism to use with */
            case smtpAUTH:
                /*
                 * SASL PLain - Authentication with server username and password
                 * Refer - http://www.ietf.org/rfc/rfc4616.txt
                 */
                _generateBase64Key((_u8*)g_Email.Username, basekey1);
                _generateBase64Key((_u8*)g_SourcePass.Password, basekey2);

                /* Send request to server for authentication */
                retVal = _sendSMTPCommand(smtpSocket, "AUTH LOGIN", NULL, buf);
                ASSERT_ON_ERROR(retVal);
                /* If response has 334, give username in base64 */
                if(buf[0] == smtp_auth_reply[0] &&
                   buf[1] == smtp_auth_reply[1] &&
                   buf[2] == smtp_auth_reply[2])
                {
                    retVal = _sendSMTPCommand(smtpSocket, basekey1, NULL, buf);
                    ASSERT_ON_ERROR(retVal);
                    /* If response has 334, give password in base64 */
                    if(buf[0] == smtp_auth_reply[0] &&
                       buf[1] == smtp_auth_reply[1] &&
                       buf[2] == smtp_auth_reply[2])
                    {
                        retVal = _sendSMTPCommand(smtpSocket, basekey2, NULL, buf);
                        ASSERT_ON_ERROR(retVal);
                    }
                }

                if(buf[0] == smtp_auth_success[0] &&
                   buf[1] == smtp_auth_success[1] &&
                   buf[2] == smtp_auth_success[2])
                {
                    /* Authentication was successful, set state to FROM */
                    g_smtpStatus = smtpFROM;
                }
                /* Else error, set state to ERROR */
                else
                {
                    g_smtpStatus = smtpERROR;
                    g_smtpErrorInfo = atAUTH;
                }
            break;

            /*
             * The SMTP transaction starts with a MAIL command which includes the
             * sender information
             * MAIL FROM:<reverse-path> [SP <mail-parameters> ] <CRLF>
             */
            case smtpFROM:
                retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_mail_from,
                                 (_u8 *)g_Email.Username, buf);
                ASSERT_ON_ERROR(retVal);

                /* If response has 250, set state to RCPT */
                if(buf[0] == smtp_ok_reply[0] &&
                   buf[1] == smtp_ok_reply[1] &&
                   buf[2] == smtp_ok_reply[2])
                {
                    g_smtpStatus = smtpRCPT;
                }
                else
                {
                    pal_Memset(email_rfc,'\0',MAX_EMAIL_RCF_LEN);
                    pal_Memcpy(email_rfc,"<",1);
                    pal_Memcpy(&email_rfc[1], g_Email.Username,
                           pal_Strlen(g_Email.Username));
                    pal_Memcpy(&email_rfc[1+pal_Strlen(g_Email.Username)],">",1);

                    retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_mail_from,
                                     email_rfc, buf);
                    ASSERT_ON_ERROR(retVal);

                    if(buf[0] == smtp_ok_reply[0] &&
                       buf[1] == smtp_ok_reply[1] &&
                       buf[2] == smtp_ok_reply[2])
                    {
                        g_smtpStatus = smtpRCPT;
                    }
                    else
                    {
                        g_smtpStatus = smtpERROR;
                        g_smtpErrorInfo = atFROM;
                    }
                }
            break;

            /* Send the destination email to the smtp server
             * RCPT TO:<forward-path> [ SP <rcpt-parameters> ] <CRLF>
             */
            case smtpRCPT:
                retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_rcpt,
                                 (_u8 *)g_Destination.Email, buf);
                ASSERT_ON_ERROR(retVal);

                /* If response has 250, set state to DATA */
                if(buf[0] == smtp_ok_reply[0] &&
                   buf[1] == smtp_ok_reply[1] &&
                   buf[2] == smtp_ok_reply[2])
                {
                    g_smtpStatus = smtpDATA;
                }
                else
                {
                    pal_Memset(email_rfc,'\0',MAX_EMAIL_RCF_LEN);
                    pal_Memcpy(email_rfc,"<",1);
                    pal_Memcpy(&email_rfc[1], g_Destination.Email,
                           pal_Strlen(g_Destination.Email));
                    pal_Memcpy(&email_rfc[1+pal_Strlen(g_Destination.Email)],">",1);
                    retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_rcpt,email_rfc, buf);
                    ASSERT_ON_ERROR(retVal);
                    /* If response has 250, set state to DATA */
                    if(buf[0] == smtp_ok_reply[0] &&
                       buf[1] == smtp_ok_reply[1] &&
                       buf[2] == smtp_ok_reply[2])
                    {
                        g_smtpStatus = smtpDATA;
                    }
                    else
                    {
                        g_smtpStatus = smtpERROR;
                        g_smtpErrorInfo = atRCPT;
                    }
                }
            break;

            /*Send the "DATA" message to the server, indicates client is ready
            * to construct the body of the email
            * DATA <CRLF>
            */
            case smtpDATA:
                retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_data, NULL, buf);
                ASSERT_ON_ERROR(retVal);
                /* If Response has 250, set state to MESSAGE */
                if(buf[0] == smtp_intermed_reply[0] &&
                buf[1] == smtp_intermed_reply[1] && buf[2] == smtp_intermed_reply[2])
                {
                    g_smtpStatus = smtpMESSAGE;
                }
                else
                {
                    g_smtpStatus = smtpERROR;
                    g_smtpErrorInfo = atDATA;
                }
            break;

            case smtpMESSAGE:
                /* Send actual Message, preceded by FROM, TO and Subject */

                /* Start with E-Mail's "Subject:" field */
                retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_subject,
                                                (_u8 *)g_Subject.Value, NULL);
                ASSERT_ON_ERROR(retVal);

                /* Add E-mail's "To:" field */
                retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_to,
                                            (_u8 *)g_Destination.Email, NULL);
                ASSERT_ON_ERROR(retVal);

                /* Add E-mail's "From:" field */
                retVal = _sendSMTPCommand(smtpSocket, (_u8 *)smtp_from,
                                                (_u8 *)g_Email.Username, NULL);
                ASSERT_ON_ERROR(retVal);

                /* Send CRLF */
                retVal = sl_Send(smtpSocket,smtp_crlf,pal_Strlen(smtp_crlf),0);
                if(retVal <= 0)
                    ASSERT_ON_ERROR(TCP_SEND_ERROR);

                /* Send body of message */
                retVal = _sendSMTPCommand(smtpSocket, message, NULL, NULL);
                ASSERT_ON_ERROR(retVal);

                /* End Message */
                retVal = _sendSMTPCommand(smtpSocket,(_u8 *)smtp_data_end,NULL,buf);
                ASSERT_ON_ERROR(retVal);

                /* Server will send 250 for successful. Move into QUIT state. */
                if(buf[0] == smtp_ok_reply[0] && buf[1] == smtp_ok_reply[1] &&
                                                        buf[2] == smtp_ok_reply[2])
                {
                    /* Disconnect from server by sending QUIT command */
                    retVal = sl_Send(smtpSocket,smtp_quit,pal_Strlen(smtp_quit),0);
                    if(retVal <= 0)
                        ASSERT_ON_ERROR(TCP_SEND_ERROR);

                    /* Close socket and reset */
                    retVal = sl_Close(smtpSocket);
                    smtpSocket = INVALID_SOCKET_DESC;
                    /* Reset the state machine */
                    g_smtpStatus = smtpINIT;
                    exit = 1;
                }
                else
                {
                    g_smtpStatus = smtpERROR;
                    g_smtpErrorInfo = atMESSAGE;
                }
            break;

            case smtpERROR:
                /* Error Handling for SMTP */
                _smtpHandleERROR(buf);
                /* Close socket and reset */
                retVal = sl_Close(smtpSocket);
                /*Reset the state machine */
                g_smtpStatus = smtpINIT;

                ASSERT_ON_ERROR(SMTP_ERROR);

            default:
                ASSERT_ON_ERROR(SMTP_INVALID_CASE);
        }
    }

    return SUCCESS;
}

/*!
    \brief This function converts the string to Base64 format needed for
           authentication

    \param[in]      input - pointer to string to be converted
    \param[out]     basekey1 - Pointer to string for base64 converted output

    \return         None

    \note

    \warning
*/
static void _generateBase64Key(_u8 *input, _u8 *basekey1)
{
    _u8 *pIn = input;

    /* Convert to base64 format  */
    ConvertToBase64(basekey1, (void *)pIn, pal_Strlen(input));
}

/*!
    \brief Performs Error Handling for SMTP State Machine

    \param[in]      servermessage - server response message

    \return         None

    \note

    \warning
*/
static void _smtpHandleERROR(_u8 *servermessage)
{
    /* Errors are handled using flags set in the smtpStateMachine */
    switch(g_smtpErrorInfo)
    {
        case atINIT:
            /* Server connection could not be established */
            CLI_Write((_u8*)"Line 686 Server connection error.\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atHELO:
            /* Server did not accept the HELO command from server */
            CLI_Write((_u8*)"Line 692 Server did not accept HELO:\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atAUTH:
            /* Server did not accept authorization credentials */
            CLI_Write((_u8*)"Line 698 Authorization unsuccessful, ");
            CLI_Write((_u8*)"Line 699 check username/password.\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atFROM:
            /* Server did not accept source email. */
            CLI_Write((_u8*)"Line 705 Email of sender not accepted by server.\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atRCPT:
            /* Server did not accept destination email */
            CLI_Write((_u8*)"Line 711 Email of recipient not accepted by server.\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atDATA:
            /* 'DATA' command to server was unsuccessful */
            CLI_Write((_u8*)"Line 717 smtp 'DATA' command not accepted by server.\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atMESSAGE:
            /* Message body could not be sent to server */
            CLI_Write((_u8*)"Line 723 Email Message was not accepted by the server.\r\n");
            CLI_Write((_u8*)servermessage);
        break;

        case atQUIT:
            /* Message could not be finalized */
            CLI_Write((_u8*)"Line 729 Connection could not be properly closed.");
            CLI_Write((_u8*)" Line 730 Message not sent.\r\n");
            CLI_Write((_u8*)servermessage);
        break;
    }
}

/*!
    \brief  Sends the SMTP command and receives the server response.
            If cmd and cmd parameter are NULL, it will only send <CR><LF>

    \param[in]      socket - Socket Descriptor
    \param[in]      cmd -    command to be send to server
    \param[in]      cmdparam - command parameter to be send
    \param[in]      respBuf -    Pointer to buffer for SMTP server response

    \return         None

    \note

    \warning
*/
static _i32 _sendSMTPCommand(_i32 socket, _u8 *cmd, _u8 * cmdparam, _u8 *respBuf)
{
    _i16 sendLen = 0;
    _i32 retVal = -1;

    pal_Memset(g_cmdBuf, 0, sizeof(g_cmdBuf));

    if(cmd != NULL)
    {
        sendLen = pal_Strlen(cmd);
        pal_Memcpy(g_cmdBuf,cmd,pal_Strlen(cmd));
    }

    if(cmdparam != NULL)
    {
        pal_Memcpy(&g_cmdBuf[sendLen], cmdparam, pal_Strlen(cmdparam));
        sendLen += pal_Strlen(cmdparam);
    }

    pal_Memcpy(&g_cmdBuf[sendLen], smtp_crlf, pal_Strlen(smtp_crlf));
    sendLen += pal_Strlen(smtp_crlf);
    retVal = sl_Send(socket, g_cmdBuf,sendLen,0);
    if(retVal <= 0)
        ASSERT_ON_ERROR(TCP_SEND_ERROR);

    if(respBuf != NULL)
    {
        pal_Memset(respBuf,0,SMTP_BUF_LEN);
        retVal = sl_Recv(socket, respBuf,SMTP_BUF_LEN,0);
        if(retVal <= 0)
            ASSERT_ON_ERROR(TCP_RECV_ERROR);
    }

    return SUCCESS;
}

//***************************************************************************
// End of Original Email file
//***************************************************************************




//***************************************************************************
// Start of Email cut and paste from Main
//***************************************************************************
//LH These 3 defines already in email.c, so copy from main not needed here
//#include "email.h"
//#include "config.h"
//#include "sl_common.h"

#define APPLICATION_VERSION "1.3.0"

#define SL_STOP_TIMEOUT        0xFF

/*
 * GLOBAL VARIABLES -- Start
 */
_u32  g_Status = 0;
/*
 * GLOBAL VARIABLES -- End
 */

volatile unsigned short usiEmailDummy = 0;

/*
 * STATIC FUNCTION DEFINITIONS -- Start
 */
static _i32 establishConnectionWithAP();
static _i32 disconnectFromAP();
static _i32 configureSimpleLinkToDefaultState();

static _i32 initializeAppVariables();
static void  displayBanner();

static _i32 setEmail();
static _i32 sendEmail();
/*
 * STATIC FUNCTION DEFINITIONS -- End
 */

//***************************************
// Start of function int LHSendEmail()
//***************************************
/*
 * Application's entry point
 */
//void LHSendEmail (void)
int LHSendEmail()
{
    usiEmailDummy = 0;
    usiEmail1OrTime2 = 1; //In the function call set to 1 for email, set to 2 for get-time


    _i32 retVal = -1;

    retVal = initializeAppVariables();
    ASSERT_ON_ERROR(retVal);

//    /* Stop WDT and initialize the system-clock of the MCU */
//    stopWDT();
//    initClk();

    /* Configure command line interface */
    CLI_Configure();

    displayBanner();

    /*
     * Following function configures the device to default state by cleaning
     * the persistent settings stored in NVMEM (viz. connection profiles &
     * policies, power policy etc)
     *
     * Applications may choose to skip this step if the developer is sure
     * that the device is in its default state at start of application
     *
     * Note that all profiles and persistent settings that were done on the
     * device will be lost
     */
    retVal = configureSimpleLinkToDefaultState();
    if(retVal < 0)
    {
        if (DEVICE_NOT_IN_STATION_MODE == retVal)
            CLI_Write(" Line 874 Failed to configure the device in its default state \n\r");

// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
        //I think better to move on, as already printed out the failure, maybe some time later it will work
//        LOOP_FOREVER();
    }

    CLI_Write(" Line 881 Device is configured in default state \n\r");

    /*
     * Initializing the CC3100 device
     * Assumption is that the device is configured in station mode already
     * and it is in its default state
     */
    retVal = sl_Start(0, 0, 0);
    if ((retVal < 0) ||
        (ROLE_STA != retVal) )
    {
        CLI_Write(" Line 892 Failed to start the device \n\r");
// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
        //I think better to move on, as already printed out the failure, maybe some time later it will work
//       LOOP_FOREVER();
    }

    CLI_Write(" Line 898 Device started as STATION \n\r");

    /* Configure the source email */
    retVal = setEmail();
    if(retVal < 0)
        LOOP_FOREVER();

    /* Connecting to WLAN AP */
    retVal = establishConnectionWithAP();
    if(retVal < 0)
    {
        CLI_Write(" Line 909 Failed to establish connection w/ an AP \n\r");
// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
        //I think better to move on, as already printed out the failure, maybe some time later it will work
//        LOOP_FOREVER();
    }

    CLI_Write(" Line 915 Connection established w/ AP and IP is acquired \n\r");

    /* Configure and send the email */
    CLI_Write(" Line 918 Sending email... \n\r"); //Jan 30, 2023 successfully get this
    retVal = sendEmail();
    if (retVal < 0)
    {
        CLI_Write(" Line 922 Device couldn't send the email \n\r");
// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
        //I think better to move on, as already printed out the failure, maybe some time later it will work
//        LOOP_FOREVER();
//        LOOP_FOREVER(); // Jan 30, 2023 put this back in as get  Device couldn't send the email \n\r
        //Did not solve the problem of not sending email and just locks at this point
    }
    else
    {
        CLI_Write(" Line 931 Email Sent successfully \n\r");
    }

    /* Disconnect from AP */
    retVal = disconnectFromAP();
    if(retVal < 0)
    {
        CLI_Write(" Line 938 Failed to disconnect from the AP \n\r");
// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
        //I think better to move on, as already printed out the failure, maybe some time later it will work
//        LOOP_FOREVER();
    }

    retVal = sl_Stop(SL_STOP_TIMEOUT);
// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
//I think better to move on, as already printed out the failure, maybe some time later it will work
//    if(retVal < 0)
//        LOOP_FOREVER();

    return  0;
} // Closing bracket for int LHSendEmail()


//***************************************
// End of function.....Start of function int LHSendEmail()
//***************************************
/*
 * ASYNCHRONOUS EVENT HANDLERS -- Start
 */

/*!
    \brief This function handles WLAN events

    \param[in]      pWlanEvent is the event passed to the handler

    \return         None

    \note

    \warning
*/
void SimpleLinkWlanEventHandler(SlWlanEvent_t *pWlanEvent)
{
    if(pWlanEvent == NULL)
    {
        CLI_Write(" Line 976 [WLAN EVENT] NULL Pointer Error \n\r");
        return;
    }

    switch(pWlanEvent->Event)
    {
        case SL_WLAN_CONNECT_EVENT:
        {
            SET_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);

            /*
             * Information about the connected AP (like name, MAC etc.) will be
             * available in 'slWlanConnectAsyncResponse_t' - Applications
             * can use it if required
             *
             * slWlanConnectAsyncResponse_t *pEventData = NULL;
             * pEventData = &pWlanEvent->EventData.STAandP2PModeWlanConnected;
             *
             */
        }
        break;

        case SL_WLAN_DISCONNECT_EVENT:
        {
            slWlanConnectAsyncResponse_t*  pEventData = NULL;

            CLR_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);
            CLR_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);

            pEventData = &pWlanEvent->EventData.STAandP2PModeDisconnected;

    if (usiEmail1OrTime2 == 1){  //In the function call set to 1 for email, set to 2 for get-time
            /* If the user has initiated 'Disconnect' request, 'reason_code' is SL_USER_INITIATED_DISCONNECTION */
            if(SL_WLAN_DISCONNECT_USER_INITIATED_DISCONNECTION == pEventData->reason_code)
            {
                CLI_Write(" Line 1011 Device disconnected from the AP on application's request \n\r");
            }
            else
            {
                CLI_Write(" Line 1015 Device disconnected from the AP on an ERROR..!! \n\r");
            }
    }
     else if (usiEmail1OrTime2 == 2){  //In the function call set to 1 for email, set to 2 for get-time
             /* If the user has initiated 'Disconnect' request, 'reason_code' is SL_USER_INITIATED_DISCONNECTION */
             if(SL_WLAN_DISCONNECT_USER_INITIATED_DISCONNECTION == pEventData->reason_code)
             {
                 CLI_Write((_u8 *)" Line 1022 Device disconnected from the AP on application's request \n\r"); //LH Line 11, last line
             }
             else
             {
                 CLI_Write((_u8 *)" Line 1026 Device disconnected from the AP on an ERROR..!! \n\r");
             }

     }
        }
        break;

        default:
        {
            CLI_Write(" Line 1035 [WLAN EVENT] Unexpected event \n\r");
        }
        break;
    }
}

/*!
    \brief This function handles events for IP address acquisition via DHCP
           indication

    \param[in]      pNetAppEvent is the event passed to the handler

    \return         None

    \note

    \warning
*/
void SimpleLinkNetAppEventHandler(SlNetAppEvent_t *pNetAppEvent)
{
    if(pNetAppEvent == NULL)
    {
        CLI_Write(" Line 1057 [NETAPP EVENT] NULL Pointer Error \n\r");
        return;
    }

    switch(pNetAppEvent->Event)
    {
        case SL_NETAPP_IPV4_IPACQUIRED_EVENT:
        {
            SET_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);

            /*
             * Information about the connected AP's IP, gateway, DNS etc.
             * will be available in 'SlIpV4AcquiredAsync_t' - Applications
             * can use it if required
             *
             * SlIpV4AcquiredAsync_t *pEventData = NULL;
             * pEventData = &pNetAppEvent->EventData.ipAcquiredV4;
             * <gateway_ip> = pEventData->gateway;
             *
             */
        }
        break;

        default:
        {
            CLI_Write(" Line 1082 [NETAPP EVENT] Unexpected event \n\r");
        }
        break;
    }
}

/*!
    \brief This function handles callback for the HTTP server events

    \param[in]      pHttpEvent - Contains the relevant event information
    \param[in]      pHttpResponse - Should be filled by the user with the
                    relevant response information

    \return         None

    \note

    \warning
*/
void SimpleLinkHttpServerCallback(SlHttpServerEvent_t *pHttpEvent,
                                  SlHttpServerResponse_t *pHttpResponse)
{
    /*
     * This application doesn't work with HTTP server - Hence these
     * events are not handled/expected here
     */
    CLI_Write(" Line 1108 [HTTP EVENT] Unexpected event \n\r");
}

/*!
    \brief This function handles general error events indication

    \param[in]      pDevEvent is the event passed to the handler

    \return         None
*/
void SimpleLinkGeneralEventHandler(SlDeviceEvent_t *pDevEvent)
{
    /*
     * Most of the general errors are not FATAL are are to be handled
     * appropriately by the application
     */
    CLI_Write(" Line 1124 [GENERAL EVENT] \n\r");
}

/*!
    \brief This function handles socket events indication

    \param[in]      pSock is the event passed to the handler

    \return         None
*/
void SimpleLinkSockEventHandler(SlSockEvent_t *pSock)
{
    if(pSock == NULL)
    {
        CLI_Write(" Line 1138 [SOCK EVENT] NULL Pointer Error \n\r");
        return;
    }

    switch( pSock->Event )
    {
        case SL_SOCKET_TX_FAILED_EVENT:
        {
            /*
            * TX Failed
            *
            * Information about the socket descriptor and status will be
            * available in 'SlSockEventData_t' - Applications can use it if
            * required
            *
            * SlSockEventData_u *pEventData = NULL;
            * pEventData = & pSock->socketAsyncEvent;
            */
            switch( pSock->socketAsyncEvent.SockTxFailData.status )
            {
                case SL_ECLOSE:
                    CLI_Write(" Line 1159[SOCK EVENT] Close socket operation failed to transmit all queued packets\n\r");
                break;


                default:
                    CLI_Write(" Line 1164 [SOCK EVENT] Unexpected event \n\r");
                break;
            }
        }
        break;

        default:
            CLI_Write(" Line 1171 [SOCK EVENT] Unexpected event \n\r");
        break;
    }
}
/*
 * ASYNCHRONOUS EVENT HANDLERS -- End
 */




/*!
    \brief This function configures the source email using parameters defined
           in "config.h" file

    \param[in]      none

    \return         none

    \note

    \warning
*/
static _i32 setEmail()
{
    _i32 retVal = -1;
    SlNetAppSourceEmail_t sourceEmailId = {0};
    SlNetAppSourcePassword_t sourceEmailPwd = {0};
    SlNetAppEmailOpt_t eMailServerSetting = {0};

    pal_Memcpy(sourceEmailId.Username,USER,pal_Strlen(USER)+1);
    retVal = sl_NetAppEmailSet(NETAPP_SOURCE_EMAIL,pal_Strlen(USER)+1,
                      (_u8*)&sourceEmailId);
    ASSERT_ON_ERROR(retVal);

    pal_Memcpy(sourceEmailPwd.Password,PASS,pal_Strlen(PASS)+1);
    retVal = sl_NetAppEmailSet(NETAPP_PASSWORD,pal_Strlen(PASS)+1,
                      (_u8*)&sourceEmailPwd);
    ASSERT_ON_ERROR(retVal);

    eMailServerSetting.Family = AF_INET;
//    eMailServerSetting.Port = GMAIL_HOST_PORT; //Line 27 in config.h    #define GMAIL_HOST_PORT         465
//    eMailServerSetting.Port = OUTLOOK_HOST_PORT; //Line  i41n config.h    #define OUTLOOK_PORT         587
    eMailServerSetting.Port = ZOHO_HOST_PORT; //Line  i41n config.h    #define OUTLOOK_PORT         587
//    eMailServerSetting.Port = VERIZON_HOST_PORT; //2-14-2023 changed to Verizon so can try LBH2015NVR
    eMailServerSetting.Ip = SL_IPV4_VAL(74,125,129,108);
    eMailServerSetting.SecurityMethod = SL_SO_SEC_METHOD_TLSV1_2; //Original 2014 to Jan 2023
    eMailServerSetting.SecurityCypher = SL_SEC_MASK_TLS_RSA_WITH_AES_256_CBC_SHA; //Original 2014 to Jan 2023

    //2-23-2023 Thomas Schmitt link said Gmail port 465 with SSL
    // CC3100 SDK1.3.0 shows about 25 Transport Security Options, Two with SSL
    // SSLV3 SSL_RSA_WITH_RC4_128_SHA
    // SSLV3 SSL_RSA_WITH_RC4_128_MD5
//    eMailServerSetting.SecurityMethod = SL_SO_SEC_METHOD_SSLV3;
//    eMailServerSetting.SecurityCypher = SL_SEC_MASK_SSL_RSA_WITH_RC4_128_SHA;
//    eMailServerSetting.SecurityCypher = SL_SEC_MASK_SSL_RSA_WITH_RC4_128_MD5;

    retVal = sl_NetAppEmailSet(NETAPP_ADVANCED_OPT,sizeof(SlNetAppEmailOpt_t),
                      (_u8*)&eMailServerSetting);
    ASSERT_ON_ERROR(retVal);

    return SUCCESS;
}

/*!
    \brief Send the email to the preconfigured email ID

    \param[in]      none

    \return         0 for success , -1 otherwise

    \note

    \warning
*/
static _i32 sendEmail()
{
    _i32 Status = -1;

    Status = sl_NetAppEmailSet(NETAPP_DEST_EMAIL,
                               pal_Strlen(DESTINATION_EMAIL),
                               (_u8 *)DESTINATION_EMAIL);
    ASSERT_ON_ERROR(Status);
    CLI_Write(" Line 1253 After...DESTINATION_EMAIL \n\r"); //Jan 30, 2023 added this
//    Status = sl_NetAppEmailSet(NETAPP_SUBJECT,
//                               pal_Strlen(EMAIL_SUBJECT),
//                               (_u8 *)EMAIL_SUBJECT);

//LH added 5-20-2020 to use my character array for the email subject
    Status = sl_NetAppEmailSet(NETAPP_SUBJECT,
                               pal_Strlen(LHemailSubject),
                               (_u8 *)LHemailSubject);


    ASSERT_ON_ERROR(Status);
    CLI_Write(" Line 1265 After...LHemailSubject \n\r"); //Jan 30, 2023 added, this is printed

//    Status = sl_NetAppEmailSet(NETAPP_MESSAGE,
//                               pal_Strlen(EMAIL_MESSAGE),
//                               (_u8 *)EMAIL_MESSAGE);

//LH added 5-20-2020 to use my character array for the email message
    Status = sl_NetAppEmailSet(NETAPP_MESSAGE,
                               pal_Strlen(LHemailBody),
                               (_u8 *)LHemailBody);


    ASSERT_ON_ERROR(Status);
    CLI_Write(" Line 1278 After...LHemailBody \n\r"); //Jan 30, 2023 added this, this IS printed

    Status = sl_NetAppEmailConnect(); //2-16-2022 looks like this function does not complete
    CLI_Write(" Line 1281 After...Status = sl_NetAppEmailConnect() \n\r"); //2-16-2022, this is NOT printed
    ASSERT_ON_ERROR(Status);
    CLI_Write(" Line 1283 After...sl_NetAppEmailConnect \n\r"); //Jan 30, 2023 added this, this is NOT printed

    Status = sl_NetAppEmailSend();
    ASSERT_ON_ERROR(Status);
    CLI_Write(" Line 1287 After...sl_NetAppEmailSend \n\r"); //Jan 30, 2023 added, this is NOT printed
    return SUCCESS;
}

/*!
    \brief This function configure the SimpleLink device in its default state. It:
           - Sets the mode to STATION
           - Configures connection policy to Auto and AutoSmartConfig
           - Deletes all the stored profiles
           - Enables DHCP
           - Disables Scan policy
           - Sets Tx power to maximum
           - Sets power policy to normal
           - Unregisters mDNS services
           - Remove all filters
    \param[in]      none

    \return         On success, zero is returned. On error, negative is returned
*/
static _i32 configureSimpleLinkToDefaultState()
{
    SlVersionFull   ver = {0};
    _WlanRxFilterOperationCommandBuff_t  RxFilterIdMask = {0};

    _u8           val = 1;
    _u8           configOpt = 0;
    _u8           configLen = 0;
    _u8           power = 0;

    _i32          retVal = -1;
    _i32          mode = -1;

    mode = sl_Start(0, 0, 0);
    ASSERT_ON_ERROR(mode);

    /* If the device is not in station-mode, try configuring it in station-mode */
    if (ROLE_STA != mode)
    {
        if (ROLE_AP == mode)
        {
            /* If the device is in AP mode, we need to wait for this event before doing anything */
            while(!IS_IP_ACQUIRED(g_Status)) { _SlNonOsMainLoopTask(); }
        }

        /* Switch to STA role and restart */
        retVal = sl_WlanSetMode(ROLE_STA);
        ASSERT_ON_ERROR(retVal);

        retVal = sl_Stop(SL_STOP_TIMEOUT);
        ASSERT_ON_ERROR(retVal);

        retVal = sl_Start(0, 0, 0);
        ASSERT_ON_ERROR(retVal);

        /* Check if the device is in station again */
        if (ROLE_STA != retVal)
        {
            /* We don't want to proceed if the device is not coming up in station-mode */
            ASSERT_ON_ERROR(DEVICE_NOT_IN_STATION_MODE);
        }
    }

    /* Get the device's version-information */
    configOpt = SL_DEVICE_GENERAL_VERSION;
    configLen = sizeof(ver);
    retVal = sl_DevGet(SL_DEVICE_GENERAL_CONFIGURATION, &configOpt, &configLen, (_u8 *)(&ver));
    ASSERT_ON_ERROR(retVal);

    /* Set connection policy to Auto + SmartConfig (Device's default connection policy) */
    retVal = sl_WlanPolicySet(SL_POLICY_CONNECTION, SL_CONNECTION_POLICY(1, 0, 0, 0, 1), NULL, 0);
    ASSERT_ON_ERROR(retVal);

    /* Remove all profiles */
    retVal = sl_WlanProfileDel(0xFF);
    ASSERT_ON_ERROR(retVal);

    /*
     * Device in station-mode. Disconnect previous connection if any
     * The function returns 0 if 'Disconnected done', negative number if already disconnected
     * Wait for 'disconnection' event if 0 is returned, Ignore other return-codes
     */
    retVal = sl_WlanDisconnect();
    if(0 == retVal)
    {
        /* Wait */
        while(IS_CONNECTED(g_Status)) { _SlNonOsMainLoopTask(); }
    }

    /* Enable DHCP client*/
    retVal = sl_NetCfgSet(SL_IPV4_STA_P2P_CL_DHCP_ENABLE,1,1,&val);
    ASSERT_ON_ERROR(retVal);

    /* Disable scan */
    configOpt = SL_SCAN_POLICY(0);
    retVal = sl_WlanPolicySet(SL_POLICY_SCAN , configOpt, NULL, 0);
    ASSERT_ON_ERROR(retVal);

    /* Set Tx power level for station mode
       Number between 0-15, as dB offset from max power - 0 will set maximum power */
    power = 0;
    retVal = sl_WlanSet(SL_WLAN_CFG_GENERAL_PARAM_ID, WLAN_GENERAL_PARAM_OPT_STA_TX_POWER, 1, (_u8 *)&power);
    ASSERT_ON_ERROR(retVal);

    /* Set PM policy to normal */
    retVal = sl_WlanPolicySet(SL_POLICY_PM , SL_NORMAL_POLICY, NULL, 0);
    ASSERT_ON_ERROR(retVal);

    /* Unregister mDNS services */
    retVal = sl_NetAppMDNSUnRegisterService(0, 0);
    ASSERT_ON_ERROR(retVal);

    /* Remove  all 64 filters (8*8) */
    pal_Memset(RxFilterIdMask.FilterIdMask, 0xFF, 8);
    retVal = sl_WlanRxFilterSet(SL_REMOVE_RX_FILTER, (_u8 *)&RxFilterIdMask,
                       sizeof(_WlanRxFilterOperationCommandBuff_t));
    ASSERT_ON_ERROR(retVal);

    retVal = sl_Stop(SL_STOP_TIMEOUT);
    ASSERT_ON_ERROR(retVal);

    retVal = initializeAppVariables();
    ASSERT_ON_ERROR(retVal);

    return retVal; /* Success */
}

/*!
    \brief Connecting to a WLAN Access point

    This function connects to the required AP (SSID_NAME).
    The function will return once we are connected and have acquired IP address

    \param[in]  None

    \return     0 on success, negative error-code on error

    \note

    \warning    If the WLAN connection fails or we don't acquire an IP address,
                We will be stuck in this function forever.
*/
static _i32 establishConnectionWithAP()
{
    if (usiEmail1OrTime2 == 1){  //In the function call set to 1 for email, set to 2 for get-time

    SlSecParams_t secParams = {0};
    _i32 retVal = 0;

    secParams.Key = PASSKEY;
    secParams.KeyLen = PASSKEY_LEN;
    secParams.Type = SEC_TYPE;

    retVal = sl_WlanConnect(SSID_NAME, pal_Strlen(SSID_NAME), 0, &secParams, 0);
    ASSERT_ON_ERROR(retVal);

    /* Wait */
    while((!IS_CONNECTED(g_Status)) || (!IS_IP_ACQUIRED(g_Status))) { _SlNonOsMainLoopTask(); }

//    return SUCCESS;
    }
    else if (usiEmail1OrTime2 == 2){  //In the function call set to 1 for email, set to 2 for get-time
    SlSecParams_t secParams = {0};
     _i32 retVal = 0;

     secParams.Key = PASSKEY;
     secParams.KeyLen = pal_Strlen(secParams.Key);
     secParams.Type = SEC_TYPE;

     retVal = sl_WlanConnect(SSID_NAME, pal_Strlen(SSID_NAME), 0, &secParams, 0);
     ASSERT_ON_ERROR(retVal);

     /* Wait */
     while((!IS_CONNECTED(g_Status)) || (!IS_IP_ACQUIRED(g_Status))) { _SlNonOsMainLoopTask(); }

//     return SUCCESS;
    }
    return SUCCESS;
}

/*!
    \brief Disconnecting from a WLAN Access point

    This function disconnects from the connected AP

    \param[in]      None

    \return         none

    \note

    \warning        If the WLAN disconnection fails, we will be stuck in this function forever.
*/
static _i32 disconnectFromAP()
{
    _i32 retVal = -1;

    /*
     * The function returns 0 if 'Disconnected done', negative number if already disconnected
     * Wait for 'disconnection' event if 0 is returned, Ignore other return-codes
     */
    retVal = sl_WlanDisconnect();
    CLI_Write(" Line 1488  in _i32 disconnectFromAP() after retVal = sl_WlanDisconnect(); \n\r");
    if(0 == retVal)
    CLI_Write(" Line 1490  in _i32 disconnectFromAP() after if(0 == retVal) \n\r");
    {
        /* Wait */
        while(IS_CONNECTED(g_Status)) { _SlNonOsMainLoopTask(); }
    }

    return SUCCESS;
}

/*!
    \brief This function initializes the application variables

    \param[in]  None

    \return     0 on success, negative error-code on error
*/
static _i32 initializeAppVariables()
{
    if (usiEmail1OrTime2 == 1){  //In the function call set to 1 for email, set to 2 for get-time
    g_Status = 0;

//    return SUCCESS;
    }
    else if (usiEmail1OrTime2 == 2){  //In the function call set to 1 for email, set to 2 for get-time
    g_Status = 0;
    pal_Memset(&appData, 0, sizeof(appData)); //This line not in email wifi call

//    return SUCCESS;
    }
    return SUCCESS;
}

/*!
    \brief This function displays the application's banner

    \param      None

    \return     None
*/
static void displayBanner()
{
    usiEmailDummy = 0;
    if (usiEmail1OrTime2 == 1){  //In the function call set to 1 for email, set to 2 for get-time
    CLI_Write("\n\r\n\r");
    CLI_Write(" Email application - Version ");
    CLI_Write(APPLICATION_VERSION);
    CLI_Write("\n\r*******************************************************************************\n\r");
    }
    else if (usiEmail1OrTime2 == 2){  //In the function call set to 1 for email, set to 2 for get-time
        usiEmailDummy = 0;
        CLI_Write((_u8 *)"\n\r\n\r");           //LH Line 0 = blank
        CLI_Write((_u8 *)" Get time application - Version ");   //LH line 1
        CLI_Write(APPLICATION_VERSION); //LH last half of line 1
        CLI_Write((_u8 *)"\n\r*******************************************************************************\n\r"); //LH Line 2 *** under top line

    }
}


//*********************************************************************************************
//****get-timeLH****get-timeLH****get-timeLH****get-timeLH****get-timeLH****get-timeLH
//****get-timeLH****get-timeLH****get-timeLH****get-timeLH****get-timeLH****get-timeLH
//****get-timeLH****get-timeLH****get-timeLH****get-timeLH****get-timeLH****get-timeLH
//*********************************************************************************************

/*
 * get-time-LH.c
 *
 *  Created on: Jul 30, 2020
 *      Author: lorne
 */

/*
 * Application Name     -   Get time
 * Application Overview -   This sample application demonstrates how
 *                          to connect to an SNTP server and request for time
 *                          information. The application processes the received
 *                          data and displays the time on console
 * Application Details  -   http://processors.wiki.ti.com/index.php/CC31xx_Get_Time_Application
 *                          doc\examples\get_time.pdf
 */

//#include "simplelink.h"  //8-1-2020 comment out as duplicate with email file above
//#include "sl_common.h"   //8-1-2020 comment out as duplicate with email file above
//#include "email.h"
//#include "get-time-LH.h"
//#include "base64.h"
//#include "config.h"
//#include "sl_common.h"
#include <stdio.h> //For printf function

#define APPLICATION_VERSION "1.3.0"

#define SL_STOP_TIMEOUT        0xFF

#define MAX_BUF_SIZE    48

#define TIME2013        3566160000       /* 43 years + 13 days of leap years */ //LH try adding 2 more days of leap years seconds
//This gives the correct date number but as expected the Day of the week is then 2 days advanced in error
//I do not use the Day of the week at this time, so OK

//#define TIME2013        3565987200       /* 43 years + 11 days of leap years */ //LH 7-29-2020; Date # is 2 days advanced
#define YEAR2013        2013
#define SEC_IN_MIN      60
#define SEC_IN_HOUR     3600
#define SEC_IN_DAY      86400

/*
 * Values for below macros shall be modified for setting the time-zone
 */

signed short GMT_TIME_ZONE_HR = -5;
//#define GMT_TIME_ZONE_HR    -5    //****Note that our time zone needs to be entered as a negative number
#define GMT_TIME_ZONE_MIN   00

/* Application specific status/error codes */
//LH 8-2-2020 added these to config.h line 70 otherwise this typedef define twice
//typedef enum{
//    DEVICE_NOT_IN_STATION_MODE = -0x7D0,        /* Choosing this number to avoid overlap w/ host-driver's error codes */
//    SNTP_SEND_ERROR = DEVICE_NOT_IN_STATION_MODE - 1,
//    SNTP_RECV_ERROR = SNTP_SEND_ERROR - 1,
//    SNTP_SERVER_RESPONSE_ERROR = SNTP_RECV_ERROR - 1,
//
//    STATUS_CODE_MAX = -0xBB8
//}e_AppStatusCodes;

/*
 * GLOBAL VARIABLES -- Start
 */
//_u32  g_Status = 0; //8-1-2020 comment out as duplicate with email file above

//LH 8-2-2020 getting errors that appData is not defined so try pasting it in email.h, I added the word typedef in front to make it the same
// Doing that generates 100's of errors, every place appData was used; took word typdef off and get a clean compile????
//struct{
//    _u32   DestinationIP;
//    _u32   elapsedSec;
//    _u32   uGeneralVar;
//    _u32   uGeneralVar1;
//
//    _u16   ccLen;
//
//    _i32   SockID;
//    _i32   sGeneralVar;
//
//    _u8 time[30];  //LH time is a structure with 30 elements so it can display a string like this Tue Oct 10 2018 11:26:13
//    _u8 *ccPtr;
//
//}appData;

volatile unsigned short usiBreakDummy = 0;
volatile unsigned short usiYearGetTime = 0;
volatile unsigned short usiMonthOfYearGetTime = 0;
volatile unsigned short usiDayOfMonthGetTime = 0;
volatile unsigned short usiDayOfWeekGetTime = 0;
volatile unsigned short usiHrsGetTime = 0;
volatile unsigned short usiMinutesGetTime = 0;
volatile unsigned short usiSecondsGetTime = 0;

 char ChrHrTens = '\0';   //Initialized to null

/*
 * GLOBAL VARIABLES -- End
 */


/*
 * CONTSTANT -- Start
 */
/*! ######################### List of SNTP servers ############################
 *! ##
 *! ##          hostname         |        IP        |       location
 *! ## ------------------------------------------------------------------------
 *! ## time-a.nist.gov           | 129.6.15.28      | NIST, Gaithersburg, Maryland
 *! ## time-b.nist.gov           | 129.6.15.29      | NIST, Gaithersburg, Maryland
 *! ## time-c.nist.gov           | 129.6.15.30      | NIST, Gaithersburg, Maryland
 *! ## nist1-macon.macon.ga.us   | 98.175.203.200   | Macon, Georgia
 *! ## 0.in.pool.ntp.org         | 123.108.225.6    | India

 *! ## For more SNTP server link visit 'http://tf.nist.gov/tf-cgi/servers.cgi'
 *! ###########################################################################
 */
//8-31-2022 did not find a, b, or c server worked any better to get the time
 const _u8 SNTPserver[30] = "time-a.nist.gov"; /* Add any one of the above servers */
//const _u8 SNTPserver[30] = "time-b.nist.gov"; /* Add any one of the above servers */
//const _u8 SNTPserver[30] = "time-c.nist.gov"; /* Add any one of the above servers */
//const _u8 SNTPserver[30] = "nist1-macon.macon.ga.us"; /* Add any one of the above servers */ //LH-7-29-2020 never got a reply

/* Tuesday is the 1st day in 2013 - the relative year */
const _u8 daysOfWeek2013[7][3] = {{"Tue"},
                                   {"Wed"},
                                   {"Thu"},
                                   {"Fri"},
                                   {"Sat"},
                                   {"Sun"},
                                   {"Mon"}};

const _u8 monthOfYear[12][3] = {{"Jan"},
                                 {"Feb"},
                                 {"Mar"},
                                 {"Apr"},
                                 {"May"},
                                 {"Jun"},
                                 {"Jul"},
                                 {"Aug"},
                                 {"Sep"},
                                 {"Oct"},
                                 {"Nov"},
                                 {"Dec"}};

const _u8 numOfDaysPerMonth[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

const _u8 digits[] = "0123456789";
/*
 * CONTSTANT -- End
 */


/*
 * STATIC FUNCTION DEFINITIONS -- Start
 */
//static _i32 establishConnectionWithAP();  //Already defined at around line 700
//static _i32 disconnectFromAP();             //Already defined at line 792
//static _i32 configureSimpleLinkToDefaultState();

//static _i32 initializeAppVariables(); //Already defined at around line 795
//static void  displayBanner(); //Already defined at line 796

//LH even in the original code these functions are not in the get-time file, must be in sl-common or some other WiFi library
static _i32 getHostIP();
static _i32 createConnection();
static _i32 getSNTPTime(_i16 gmt_hr, _i16 gmt_min);
static _u16 itoa(_i16 cNum, _u8 *cString);

/*
 * STATIC FUNCTION DEFINITIONS -- End
 */



/*
 * Application's entry point
 */
int LHGetTime ()
//int main(int argc, char** argv)
{
    //usiGMTOffsetHrSet is set in mode = Settings, so End user can set GMT for their Time zone and Day light saving time
    GMT_TIME_ZONE_HR = (-1 * usiGMTOffsetHrSet);

    usiEmail1OrTime2 = 2; //In the function call set to 1 for email, set to 2 for get-time
    _i32 retVal = -1;

    retVal = initializeAppVariables();
    ASSERT_ON_ERROR(retVal);

    /* Stop WDT and initialize the system-clock of the MCU */
//    stopWDT();
//    initClk();

    /* Configure command line interface */
    CLI_Configure();

    displayBanner();

    /*
     * Following function configures the device to default state by cleaning
     * the persistent settings stored in NVMEM (viz. connection profiles &
     * policies, power policy etc)
     *
     * Applications may choose to skip this step if the developer is sure
     * that the device is in its default state at start of application
     *
     * Note that all profiles and persistent settings that were done on the
     * device will be lost
     */
    retVal = configureSimpleLinkToDefaultState();
    if(retVal < 0)
    {
        if (DEVICE_NOT_IN_STATION_MODE == retVal)
            CLI_Write((_u8 *)" Line 1757 Failed to configure the device in its default state \n\r");
        // 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
                //I think better to move on, as already printed out the failure, maybe some time later it will work
        //LOOP_FOREVER();
    }

    CLI_Write((_u8 *)" Line 1774 Device is configured in default state \n\r");    //LH line 3

    /*
     * Initializing the CC3100 device
     * Assumption is that the device is configured in station mode already
     * and it is in its default state
     */
    retVal = sl_Start(0, 0, 0);
    if ((retVal < 0) ||
        (ROLE_STA != retVal) )
    {
        CLI_Write((_u8 *)" Line 1785 Failed to start the device \n\r");
        // 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
                //I think better to move on, as already printed out the failure, maybe some time later it will work
        //LOOP_FOREVER();
    }

    CLI_Write((_u8 *)" Line 1791 Device started as STATION \n\r"); //LH line 4

    /* Connecting to WLAN AP */
    retVal = establishConnectionWithAP();
    if(retVal < 0)
    {
        CLI_Write((_u8 *)" Line 1797 Failed to establish connection w/ an AP \n\r");
        // 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
                //I think better to move on, as already printed out the failure, maybe some time later it will work
        //LOOP_FOREVER();
    }

    CLI_Write(" Line 1803 Connection established w/ AP and IP is acquired \n\r"); //LH Line 5

//8-2-2020 8:30 PM Does complete the above statement, then no further

    retVal = getHostIP();
    CLI_Write(" Line 1808 LH Just after get HostIP \n\r");

    if(retVal < 0)
    {
        CLI_Write((_u8 *)" Line 1812 Unable to get host IP\n\r\n\r");
        // 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
                //I think better to move on, as already printed out the failure, maybe some time later it will work
        //LOOP_FOREVER();
    }
    appData.SockID = createConnection();
    CLI_Write(" Line 1818 LH Just after createConnection \n\r");

// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
//I think better to move on, as already printed out the failure, maybe some time later it will work
//    if(appData.SockID < 0)
        //LOOP_FOREVER();

        CLI_Write(" Line 1825 LH Just BEFORE getSNTPTTime... \n\r");

    retVal = getSNTPTime(GMT_TIME_ZONE_HR,GMT_TIME_ZONE_MIN);

       CLI_Write(" Line 1829 LH Just AFTER getSNTPTTime... \n\r");

// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
//I think better to move on, as already printed out the failure, maybe some time later it will work
//    if(retVal < 0)
        //LOOP_FOREVER();

    usiBreakDummy = 0;
    retVal = sl_Close(appData.SockID);

// 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
//I think better to move on, as already printed out the failure, maybe some time later it will work
//    if(retVal < 0)
        //LOOP_FOREVER();

    usiBreakDummy = 0;
    retVal = disconnectFromAP();
    if(retVal < 0)
    {
        CLI_Write((_u8 *)" Line 1848 Failed to disconnect from the AP \n\r");
        // 5-24-2020, I don't see the value of LOOP_FOREVER(); as this just hangs the program forever at this point
                //I think better to move on, as already printed out the failure, maybe some time later it will work
        //LOOP_FOREVER();
    }

//LH added 7-31-2020
    usiBreakDummy = 0;
    retVal = sl_Stop(SL_STOP_TIMEOUT);

    return 0;
}


//******End of Application Function called from main*************




/*
 * ASYNCHRONOUS EVENT HANDLERS -- Start
 */
//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function handles WLAN events
//
//    \param[in]      pWlanEvent is the event passed to the handler
//
//    \return         None
//
//    \note
//
//    \warning
//*/
//void SimpleLinkWlanEventHandler(SlWlanEvent_t *pWlanEvent)
//{
//    if(pWlanEvent == NULL)
//    {
//        CLI_Write(" [WLAN EVENT] NULL Pointer Error \n\r");
//        return;
//    }
//
//    switch(pWlanEvent->Event)
//    {
//        case SL_WLAN_CONNECT_EVENT:
//        {
//            SET_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);
//
//            /*
//             * Information about the connected AP (like name, MAC etc) will be
//             * available in 'slWlanConnectAsyncResponse_t' - Applications
//             * can use it if required
//             *
//             * slWlanConnectAsyncResponse_t *pEventData = NULL;
//             * pEventData = &pWlanEvent->EventData.STAandP2PModeWlanConnected;
//             *
//             */
//        }
//        break;
//
//        case SL_WLAN_DISCONNECT_EVENT:
//        {
//            slWlanConnectAsyncResponse_t*  pEventData = NULL;
//
//            CLR_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);
//            CLR_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);
//
//            pEventData = &pWlanEvent->EventData.STAandP2PModeDisconnected;
//
//            /* If the user has initiated 'Disconnect' request, 'reason_code' is SL_USER_INITIATED_DISCONNECTION */
//            if(SL_WLAN_DISCONNECT_USER_INITIATED_DISCONNECTION == pEventData->reason_code)
//            {
//                CLI_Write((_u8 *)" Device disconnected from the AP on application's request \n\r"); //LH Line 11, last line
//            }
//            else
//            {
//                CLI_Write((_u8 *)" Device disconnected from the AP on an ERROR..!! \n\r");
//            }
//        }
//        break;
//
//        default:
//        {
//            CLI_Write((_u8 *)" [WLAN EVENT] Unexpected event \n\r");
//        }
//        break;
//    }
//}


//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function handles events for IP address acquisition via DHCP
//           indication
//
//    \param[in]      pNetAppEvent is the event passed to the handler
//
//    \return         None
//
//    \note
//
//    \warning
//*/
//void SimpleLinkNetAppEventHandler(SlNetAppEvent_t *pNetAppEvent)
//{
//    if(pNetAppEvent == NULL)
//    {
//        CLI_Write(" [NETAPP EVENT] NULL Pointer Error \n\r");
//        return;
//    }
//
//    switch(pNetAppEvent->Event)
//    {
//        case SL_NETAPP_IPV4_IPACQUIRED_EVENT:
//        {
//            SET_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);
//
//            /*
//             * Information about the connected AP's IP, gateway, DNS etc
//             * will be available in 'SlIpV4AcquiredAsync_t' - Applications
//             * can use it if required
//             *
//             * SlIpV4AcquiredAsync_t *pEventData = NULL;
//             * pEventData = &pNetAppEvent->EventData.ipAcquiredV4;
//             * <gateway_ip> = pEventData->gateway;
//             *
//             */
//        }
//        break;
//
//        default:
//        {
//            CLI_Write((_u8 *)" [NETAPP EVENT] Unexpected event \n\r");
//        }
//        break;
//    }
//}

//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function handles callback for the HTTP server events
//
//    \param[in]      pHttpEvent - Contains the relevant event information
//    \param[in]      pHttpResponse - Should be filled by the user with the
//                    relevant response information
//
//    \return         None
//
//    \note
//
//    \warning
//*/
//void SimpleLinkHttpServerCallback(SlHttpServerEvent_t *pHttpEvent,
//                                  SlHttpServerResponse_t *pHttpResponse)
//{
//    /*
//     * This application doesn't work with HTTP server - Hence these
//     * events are not handled here
//     */
//    CLI_Write((_u8 *)" [HTTP EVENT] Unexpected event \n\r");
//}

//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function handles general error events indication
//
//    \param[in]      pDevEvent is the event passed to the handler
//
//    \return         None
//*/
//void SimpleLinkGeneralEventHandler(SlDeviceEvent_t *pDevEvent)
//{
//    /*
//     * Most of the general errors are not FATAL are to be handled
//     * appropriately by the application
//     */
//    CLI_Write((_u8 *)" [GENERAL EVENT] \n\r");
//}

//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function handles socket events indication
//
//    \param[in]      pSock is the event passed to the handler
//
//    \return         None
//*/
//void SimpleLinkSockEventHandler(SlSockEvent_t *pSock)
//{
//    if(pSock == NULL)
//    {
//        CLI_Write(" [SOCK EVENT] NULL Pointer Error \n\r");
//        return;
//    }
//
//    switch( pSock->Event )
//    {
//        case SL_SOCKET_TX_FAILED_EVENT:
//        {
//            /*
//            * TX Failed
//            *
//            * Information about the socket descriptor and status will be
//            * available in 'SlSockEventData_t' - Applications can use it if
//            * required
//            *
//            * SlSockEventData_u *pEventData = NULL;
//            * pEventData = & pSock->socketAsyncEvent;
//            */
//            switch( pSock->socketAsyncEvent.SockTxFailData.status )
//            {
//                case SL_ECLOSE:
//                    CLI_Write((_u8 *)" [SOCK EVENT] Close socket operation failed to transmit all queued packets\n\r");
//                break;
//
//
//                default:
//                    CLI_Write((_u8 *)" [SOCK EVENT] Unexpected event \n\r");
//                break;
//            }
//        }
//        break;
//
//        default:
//            CLI_Write((_u8 *)" [SOCK EVENT] Unexpected event \n\r");
//        break;
//    }
//}
/*
 * ASYNCHRONOUS EVENT HANDLERS -- End
 */



/*!
    \brief Convert integer to ASCII in decimal base

    \param[in]      cNum - integer number to convert

    \param[OUT]     cString - output string

    \return         number of ASCII characters

    \warning
*/
static _u16 itoa(_i16 cNum, _u8 *cString)
{
    _u16 length = 0;
    _u8* ptr = NULL;
    _i16 uTemp = cNum;

    /* value 0 is a special case */
    if (cNum == 0)
    {
        length = 1;
        *cString = '0';

        return length;
    }

    /* Find out the length of the number, in decimal base */
    length = 0;
    while (uTemp > 0)
    {
        uTemp /= 10;
        length++;
    }

    /* Do the actual formatting, right to left */
    uTemp = cNum;
    ptr = cString + length;
    while (uTemp > 0)
    {
        --ptr;
        *ptr = digits[uTemp % 10];
        uTemp /= 10;
    }

    return length;
}

/*!
    \brief Get the required data from the server.

    \param[in]      gmt_hr - GMT offset hours

    \param[in]      gmt_min - GMT offset minutes

    \return         0 on success, -ve otherwise

    \warning
*/
static _i32 getSNTPTime(_i16 gmt_hr, _i16 gmt_min)
{
    /*
                                NTP Packet Header:


           0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9  0  1
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |LI | VN  |Mode |    Stratum    |     Poll      |   Precision    |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                          Root  Delay                           |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                       Root  Dispersion                         |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                     Reference Identifier                       |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                    Reference Time-stamp (64)                    |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                    Originate Time-stamp (64)                    |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                     Receive Time-stamp (64)                     |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                     Transmit Time-stamp (64)                    |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                 Key Identifier (optional) (32)                 |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          |                                                                |
          |                                                                |
          |                 Message Digest (optional) (128)                |
          |                                                                |
          |                                                                |
          +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    */

    SlSockAddrIn_t  LocalAddr;
    SlSockAddr_t Addr;

    _u8    dataBuf[MAX_BUF_SIZE];
    _i32     retVal = -1;
    _i16     AddrSize = 0;

    /* For time zone with negative GMT value, change minutes to negative for
     * computation */
    if(gmt_hr < 0 && gmt_min > 0)
        gmt_min = gmt_min * (-1);

    sl_Memset(dataBuf, 0, sizeof(dataBuf));
    dataBuf[0] = '\x1b';

    Addr.sa_family = AF_INET;
    /* the source port */
    Addr.sa_data[0] = 0x00;
    Addr.sa_data[1] = 0x7B;    /* 123 */
    Addr.sa_data[2] = (_u8)((appData.DestinationIP >> 24) & 0xff);
    Addr.sa_data[3] = (_u8)((appData.DestinationIP >> 16) & 0xff);
    Addr.sa_data[4] = (_u8)((appData.DestinationIP >> 8) & 0xff);
    Addr.sa_data[5] = (_u8) (appData.DestinationIP & 0xff);

    retVal = sl_SendTo(appData.SockID, dataBuf, sizeof(dataBuf), 0,
                     &Addr, sizeof(Addr));
    if (retVal != sizeof(dataBuf))
    {
        /* could not send SNTP request */
        CLI_Write((_u8 *)" Line 2211 Device couldn't send SNTP request\n\r\n\r");
        ASSERT_ON_ERROR(SNTP_SEND_ERROR);
    }

    AddrSize = sizeof(SlSockAddrIn_t);
    LocalAddr.sin_family = SL_AF_INET;
    LocalAddr.sin_port = 0;
    LocalAddr.sin_addr.s_addr = 0;

    retVal = sl_Bind(appData.SockID,(SlSockAddr_t *)&LocalAddr, AddrSize);
    if(retVal < 0)
        ASSERT_ON_ERROR(retVal);

    retVal = sl_RecvFrom(appData.SockID, dataBuf, sizeof(dataBuf), 0,
                       (SlSockAddr_t *)&LocalAddr,  (SlSocklen_t*)&AddrSize);
    if (retVal <= 0)
    {
        CLI_Write((_u8 *)" Line 2228 Device couldn't receive time information \n\r");
        ASSERT_ON_ERROR(SNTP_RECV_ERROR);
    }

    if ((dataBuf[0] & 0x7) != 4)    /* expect only server response */
    {
        /* MODE is not server, abort */
        CLI_Write((_u8 *)" Line 2235 Device is expecting response from server only!\n\r");
        ASSERT_ON_ERROR(SNTP_SERVER_RESPONSE_ERROR);
    }
    else
    {
        _u8 index;

        appData.elapsedSec = dataBuf[40];
        appData.elapsedSec <<= 8;
        appData.elapsedSec += dataBuf[41];
        appData.elapsedSec <<= 8;
        appData.elapsedSec += dataBuf[42];
        appData.elapsedSec <<= 8;
        appData.elapsedSec += dataBuf[43];

        appData.elapsedSec -= TIME2013;

        /* correct the time zone */
        appData.elapsedSec += (gmt_hr * SEC_IN_HOUR);
        appData.elapsedSec += (gmt_min * SEC_IN_MIN);

        appData.ccPtr = &appData.time[0];

        /* day */
//        appData.sGeneralVar = appData.elapsedSec/SEC_IN_DAY;
        appData.sGeneralVar = (appData.elapsedSec + 172800) /SEC_IN_DAY; //LH Day of week 2 days behind due to leap years

        //This fixed the Day of week, but then re-broke the Date number
        pal_Memcpy(appData.ccPtr, daysOfWeek2013[appData.sGeneralVar%7], 3);
//Day of the week
        usiDayOfWeekGetTime = appData.sGeneralVar%7;
/* Tuesday is the 1st day in 2013 - the relative year */
//const _u8 daysOfWeek2013[7][3] = {{"Tue"},    // 0 = Tuesday
//                                   {"Wed"},   // 1 = Wednesday
//                                   {"Thu"},   // 2 = Thursday
//                                   {"Fri"},   // 3 = Friday
//                                   {"Sat"},   // 4 = Saturday
//                                   {"Sun"},   // 5 = Sunday
//                                   {"Mon"}};  // 6 = Monday
        appData.sGeneralVar = appData.elapsedSec /SEC_IN_DAY; //LH restore the sGeneralVar so date calculates correctly

        appData.ccPtr += 3;
        *appData.ccPtr++ = '\x20';

        /* month */
        appData.sGeneralVar %= 365;
        for (index = 0; index < 12; index++)
        {
            appData.sGeneralVar -= numOfDaysPerMonth[index];
            if (appData.sGeneralVar < 0)
                break;
        }

        pal_Memcpy(appData.ccPtr, monthOfYear[index], 3);
//Month of the Year
        usiMonthOfYearGetTime = index + 1; //See Reference table at top of Get time section, Array starts at zero, so if want normal mth # add 1
        appData.ccPtr += 3;
        *appData.ccPtr++ = '\x20';

        /* date */
        /* restore the day in current month*/
        appData.sGeneralVar += numOfDaysPerMonth[index];
        appData.ccLen = itoa(appData.sGeneralVar + 1, appData.ccPtr); //Calls function itoa integer to ASCII decimal
        //LH Convert integer to ASCII in decimal base; param[in] cNum - integer number to convert
        //param[OUT] cString - output string; \return  number of ASCII characters
        //So at this point uGeneralVar must be a 2 digit integer that contains the day of the month
//Day of the Month
        usiDayOfMonthGetTime = appData.sGeneralVar +1; //note this is s not u  GeneralVar; Had to add the +1 like above, not sure why but do
        usiBreakDummy = 0;

        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = '\x20';

        /* year */
        /* number of days since beginning of 2013 */
        appData.uGeneralVar = appData.elapsedSec/SEC_IN_DAY;
        appData.uGeneralVar /= 365;
        appData.ccLen = itoa(YEAR2013 + appData.uGeneralVar , appData.ccPtr); //So this would be 2013 + 7 = 2020, so uGeneralVar must = # incr yr after 2013
        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = '\x20';
//Year
        usiYearGetTime = YEAR2013 + appData.uGeneralVar;

        /* time */
        appData.uGeneralVar = appData.elapsedSec%SEC_IN_DAY;
        /* number of seconds per hour */
        appData.uGeneralVar1 = appData.uGeneralVar%SEC_IN_HOUR;
        appData.uGeneralVar /= SEC_IN_HOUR;               /* number of hours */
        appData.ccLen = itoa(appData.uGeneralVar, appData.ccPtr);   //LH believe Hrs to Char
        //So at this point uGeneralVar must be a 2 digit integer that contains hours
        usiBreakDummy = 0;
//Hours
        usiHrsGetTime = appData.uGeneralVar;
        usiBreakDummy = 0;

        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = ':';                                     //LH believe colon between Hr and Min
        /* number of minutes per hour */
        appData.uGeneralVar = appData.uGeneralVar1/SEC_IN_MIN;
        /* number of seconds per minute */
        appData.uGeneralVar1 %= SEC_IN_MIN;
        appData.ccLen = itoa(appData.uGeneralVar, appData.ccPtr);   //LH believe minutes to Char
        //So at this point uGeneralVar must be a 2 digit integer that contains minutes
//Minutes
        usiMinutesGetTime = appData.uGeneralVar;
        usiBreakDummy = 0;

        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = ':';                                     //LH believe colon between Min and Sec
        appData.ccLen = itoa(appData.uGeneralVar1, appData.ccPtr);  //LH believe seconds to Char
        //So at this point uGeneralVar1 must be a 2 digit integer that contains seconds
//Seconds
        usiSecondsGetTime = appData.uGeneralVar1;
        usiBreakDummy = 0;

        appData.ccPtr += appData.ccLen;
        *appData.ccPtr++ = '\x20';

        *appData.ccPtr++ = '\0';

        CLI_Write((_u8 *)"\r\n Server "); //LH Line 7
        CLI_Write((_u8 *)SNTPserver);
        CLI_Write((_u8 *)" Line 2346 has responded with time information");
        CLI_Write((_u8 *)"\n\r\r\n ");  //LH Line 8 which is blank
        CLI_Write((_u8 *)appData.time);  //LH when look at time structure it only uses 23 of the 30 elements in the structure; Line 9 with the time
        CLI_Write((_u8 *)"\n\r\r\n");   //LH line 10 which is a blank line

        //time element 16 HrTens, element 17 HrUnits; element 19 MinTens, element 20 MinUnits, element 22 SecTens, element 23 SecUnits
        //LH added 7-31-2020
        //CLI is set up for characters, so it prints the characters but not the integers
//        CLI_Write((_u8 *)"\r\n usiDayOfMonth:  ");
//        CLI_Write((_u8 *)usiDayOfMonth);
//        CLI_Write((_u8 *)"\r\n usiHrs:  ");
//        CLI_Write((_u8 *)usiHrs);
//        CLI_Write((_u8 *)"\r\n usiMinutes:  ");
//        CLI_Write((_u8 *)usiMinutes);
//        CLI_Write((_u8 *)"\r\n usiDayOfMonth:  ");
//        CLI_Write((_u8 *)usiSeconds);
//printf compiles and runs but no seen on the console, not sure were it has gone to
//        printf("\r\n usiDayOfMonth: %d", usiDayOfMonth);
//        printf("\r\n usiHrs: %d", usiHrs);
//        printf("\r\n usiMinutes: %d", usiMinutes);
//        printf("\r\n usiSeconds: %d", usiSeconds);

    }

    return SUCCESS;
}

/*!
    \brief Create UDP socket to communicate with server.

    \param[in]      none

    \return         Socket descriptor for success otherwise negative

    \warning
*/
static _i32 createConnection()
{
    _i32 sd = 0;

    sd = sl_Socket(SL_AF_INET, SL_SOCK_DGRAM, IPPROTO_UDP);
    if( sd < 0 )
    {
        CLI_Write((_u8 *)"Error creating socket\n\r\n\r");
    }

    return sd;
}

/*!
    \brief Gets the Server IP address

    \param[in]      none

    \return         zero for success and -1 for error

    \warning
*/
static _i32 getHostIP()
{
    _i32 status = 0;
    appData.DestinationIP = 0;

    status = sl_NetAppDnsGetHostByName((_i8*)SNTPserver, pal_Strlen(SNTPserver),
                                       &appData.DestinationIP, SL_AF_INET);
    ASSERT_ON_ERROR(status);

    return SUCCESS;
}


//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function configure the SimpleLink device in its default state. It:
//           - Sets the mode to STATION
//           - Configures connection policy to Auto and AutoSmartConfig
//           - Deletes all the stored profiles
//           - Enables DHCP
//           - Disables Scan policy
//           - Sets Tx power to maximum
//           - Sets power policy to normal
//           - Unregisters mDNS services
//           - Remove all filters
//
//    \param[in]      none
//
//    \return         On success, zero is returned. On error, negative is returned
//*/
//static _i32 configureSimpleLinkToDefaultState()
//{
//    SlVersionFull   ver = {0};
//    _WlanRxFilterOperationCommandBuff_t  RxFilterIdMask = {0};
//
//    _u8           val = 1;
//    _u8           configOpt = 0;
//    _u8           configLen = 0;
//    _u8           power = 0;
//
//    _i32          retVal = -1;
//    _i32          mode = -1;
//
//    mode = sl_Start(0, 0, 0);
//    ASSERT_ON_ERROR(mode);
//
//    /* If the device is not in station-mode, try configuring it in station-mode */
//    if (ROLE_STA != mode)
//    {
//        if (ROLE_AP == mode)
//        {
//            /* If the device is in AP mode, we need to wait for this event before doing anything */
//            while(!IS_IP_ACQUIRED(g_Status)) { _SlNonOsMainLoopTask(); }
//        }
//
//        /* Switch to STA role and restart */
//        retVal = sl_WlanSetMode(ROLE_STA);
//        ASSERT_ON_ERROR(retVal);
//
//        retVal = sl_Stop(SL_STOP_TIMEOUT);
//        ASSERT_ON_ERROR(retVal);
//
//        retVal = sl_Start(0, 0, 0);
//        ASSERT_ON_ERROR(retVal);
//
//        /* Check if the device is in station again */
//        if (ROLE_STA != retVal)
//        {
//            /* We don't want to proceed if the device is not coming up in station-mode */
//            ASSERT_ON_ERROR(DEVICE_NOT_IN_STATION_MODE);
//        }
//    }
//
//    /* Get the device's version-information */
//    configOpt = SL_DEVICE_GENERAL_VERSION;
//    configLen = sizeof(ver);
//    retVal = sl_DevGet(SL_DEVICE_GENERAL_CONFIGURATION, &configOpt, &configLen, (_u8 *)(&ver));
//    ASSERT_ON_ERROR(retVal);
//
//    /* Set connection policy to Auto + SmartConfig (Device's default connection policy) */
//    retVal = sl_WlanPolicySet(SL_POLICY_CONNECTION, SL_CONNECTION_POLICY(1, 0, 0, 0, 1), NULL, 0);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Remove all profiles */
//    retVal = sl_WlanProfileDel(0xFF);
//    ASSERT_ON_ERROR(retVal);
//
//    /*
//     * Device in station-mode. Disconnect previous connection if any
//     * The function returns 0 if 'Disconnected done', negative number if already disconnected
//     * Wait for 'disconnection' event if 0 is returned, Ignore other return-codes
//     */
//    retVal = sl_WlanDisconnect();
//    if(0 == retVal)
//    {
//        /* Wait */
//        while(IS_CONNECTED(g_Status)) { _SlNonOsMainLoopTask(); }
//    }
//
//    /* Enable DHCP client*/
//    retVal = sl_NetCfgSet(SL_IPV4_STA_P2P_CL_DHCP_ENABLE,1,1,&val);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Disable scan */
//    configOpt = SL_SCAN_POLICY(0);
//    retVal = sl_WlanPolicySet(SL_POLICY_SCAN , configOpt, NULL, 0);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Set Tx power level for station mode
//       Number between 0-15, as dB offset from max power - 0 will set maximum power */
//    power = 0;
//    retVal = sl_WlanSet(SL_WLAN_CFG_GENERAL_PARAM_ID, WLAN_GENERAL_PARAM_OPT_STA_TX_POWER, 1, (_u8 *)&power);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Set PM policy to normal */
//    retVal = sl_WlanPolicySet(SL_POLICY_PM , SL_NORMAL_POLICY, NULL, 0);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Unregister mDNS services */
//    retVal = sl_NetAppMDNSUnRegisterService(0, 0);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Remove  all 64 filters (8*8) */
//    pal_Memset(RxFilterIdMask.FilterIdMask, 0xFF, 8);
//    retVal = sl_WlanRxFilterSet(SL_REMOVE_RX_FILTER, (_u8 *)&RxFilterIdMask,
//                       sizeof(_WlanRxFilterOperationCommandBuff_t));
//    ASSERT_ON_ERROR(retVal);
//
//    retVal = sl_Stop(SL_STOP_TIMEOUT);
//    ASSERT_ON_ERROR(retVal);
//
//    retVal = initializeAppVariables();
//    ASSERT_ON_ERROR(retVal);
//
//    return retVal; /* Success */
//}


//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief Connecting to a WLAN Access point
//
//    This function connects to the required AP (SSID_NAME).
//    The function will return once we are connected and have acquired IP address
//
//    \param[in]  None
//
//    \return     0 on success, negative error-code on error
//
//    \note
//
//    \warning    If the WLAN connection fails or we don't acquire an IP address,
//                We will be stuck in this function forever.
//*/
//static _i32 establishConnectionWithAP()
//{
//    SlSecParams_t secParams = {0};
//    _i32 retVal = 0;
//
//    secParams.Key = PASSKEY;
//    secParams.KeyLen = pal_Strlen(secParams.Key);
//    secParams.Type = SEC_TYPE;
//
//    retVal = sl_WlanConnect(SSID_NAME, pal_Strlen(SSID_NAME), 0, &secParams, 0);
//    ASSERT_ON_ERROR(retVal);
//
//    /* Wait */
//    while((!IS_CONNECTED(g_Status)) || (!IS_IP_ACQUIRED(g_Status))) { _SlNonOsMainLoopTask(); }
//
//    return SUCCESS;
//}


//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief Disconnecting from a WLAN Access point
//
//    This function disconnects from the connected AP
//
//    \param[in]      None
//
//    \return         none
//
//    \note
//
//    \warning        If the WLAN disconnection fails, we will be stuck in this function forever.
//*/
//static _i32 disconnectFromAP()
//{
//    _i32 retVal = -1;
//
//    /*
//     * The function returns 0 if 'Disconnected done', negative number if already disconnected
//     * Wait for 'disconnection' event if 0 is returned, Ignore other return-codes
//     */
//    retVal = sl_WlanDisconnect();
//    if(0 == retVal)
//    {
//        /* Wait */
//        while(IS_CONNECTED(g_Status)) { _SlNonOsMainLoopTask(); }
//    }
//
//    return SUCCESS;
//}


//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function initializes the application variables
//
//    \param[in]  None
//
//    \return     0 on success, negative error-code on error
//*/
//unsigned short usiEmail1OrTime2 = 0;
//static _i32 initializeAppVariables()
//{
//    g_Status = 0;
//    pal_Memset(&appData, 0, sizeof(appData)); //This line not in email wifi call
//
//    return SUCCESS;
//}

//LH-8-1-2020 copied from get-time-LH, duplicate in email.c above
///*!
//    \brief This function displays the application's banner
//
//    \param      None
//
//    \return     None
//*/
//static void displayBanner()
//{
//    CLI_Write((_u8 *)"\n\r\n\r");           //LH Line 0 = blank
//    CLI_Write((_u8 *)" Get time application - Version ");   //LH line 1
//    CLI_Write(APPLICATION_VERSION); //LH last half of line 1
//    CLI_Write((_u8 *)"\n\r*******************************************************************************\n\r"); //LH Line 2 *** under top line
//}

